==> cdm-core-1.0-RELEASE-sources.jar: online/sanen/cdm/handel/ResultHandel.java <==
package online.sanen.cdm.handel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import com.mhdt.toolkit.Bean;
import com.mhdt.toolkit.Collections;

import online.sanen.cdm.basic.QueryException;
import online.sanen.cdm.basic.QueryType;
import online.sanen.cdm.basic.ResultType;
import online.sanen.cdm.basic.Structure;

/**
 * The execution result
 * <pre>
 * @author LazyToShow
 * 
 * Date: 2017/10/21 
 * Time: 23:19
 * </pre>
 */
public class ResultHandel extends DefaultHandel {

	@Override
	public Object handel(Structure structure, Object product) throws QueryException {

		// Initialize the underlying data
		String sql = structure.getSql().toString();
		Collection<Object> paramers = structure.getParamers().values();
		QueryType queryType = structure.getQueryType();
		JdbcTemplate template = structure.getJdbcTemplate();

		// Modify the operating
		if (queryType != QueryType.select) {
			if (structure.isCache())
				noticeUpdate(structure);
			return template.update(sql, paramers.toArray());
		}

		// Query operation
		ResultType resultType = structure.getResultType();

		Object result = null;

		// Try to read the results from the cache
		if (structure.isCache() && (result = tryReadFromCache(structure)) != null) {
			return processCacheResult(result, resultType);
		}

		
		try {
			switch (resultType) {

			case Int:
				result = queryForInt(sql, template, paramers);
				break;
			case String:
				result = queryForString(sql, template, paramers);
				break;
			case List:
				result = queryForList(structure, sql, template, paramers);
				break;
			case Object:
				result = queryForObject(structure, sql, template, paramers);
				break;
			case Bean:
				result = queryForBean(structure, sql, template, paramers);
				break;
			case Beans:
				result = queryForBeans(structure, sql, template, paramers);
				break;
			}

			// The result set requires manual limitations ?
			result = processLimit(result, structure);
			// Add the select cache
			if (structure.isCache())
				noticeAdd(structure, result);

			return result;

		} catch (Exception e) {
			throw new QueryException(e.getMessage() + formatSql(structure), e);
		}

	}

	/**
	 * Resolve different problems with the same SQL result set<br>
	 * For example, the first query result is a list, and the second query result is
	 * object The second result is to be processed in the cache result
	 * 
	 * @param result
	 * @param resultType
	 * @return
	 */
	private Object processCacheResult(Object result, ResultType resultType) {

		if (result instanceof List && (resultType.equals(ResultType.Object) || resultType.equals(ResultType.Bean))) {
			return ((List<?>) result).get(0);
		}

		return result;
	}

	/**
	 * The result and the second limit,SQL that is not supported by the database
	 * Here we are implementing the interface manually
	 * 
	 * @param result
	 * @param structure
	 */
	private Object processLimit(Object result, Structure structure) {
		// If result is not list type,need not process
		if (!(result instanceof List))
			return result;

		// If there is a limit operation but SQL does not support, simulate limit
		if (structure.hasLimitAble() && !structure.isSupportLimitAble()) {
			
			// If the limit second parameter is null converted to -1, the number of access
			// is not limited
			result = Collections.limit((List<?>) result, structure.getLimit()[0],
					structure.getLimit()[1] == null ? -1 : structure.getLimit()[1]);
		}

		return result;
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForList(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		List<Map<String, Object>> list = queryForMaps(sql, template, paramers);

		if (list == null || list.isEmpty())
			return null;

		if (structure.getEntry_class() == null) {
			return processList(list);
		} else {
			return Bean.parse(structure.getEntry_class(), list);
		}
	}

	private Object processList(List<Map<String, Object>> list) {
		if (list.get(0).keySet().size() > 1)
			return list;
		List<Object> result = new ArrayList<Object>();
		for (Map<String, Object> map : list) {
			result.add(map.values().toArray()[0]);
		}

		return result;
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForBeans(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		List<Map<String, Object>> list = queryForMaps(sql, template, paramers);
		if (list == null || list.isEmpty())
			return null;

		return Bean.parse(structure.getEntry_class(), list);
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForBean(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		Map<String, Object> map = queryForMap(sql, template, paramers);
		if (map == null || map.isEmpty())
			return null;
		return Bean.parse(structure.getEntry_class(), map);
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForObject(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		List<Map<String, Object>> maps = queryForMaps(sql, template, paramers);
		if (maps == null || maps.isEmpty())
			return null;

		Map<String, Object> map = null;
		if (maps.size() > 0)
			map = maps.get(0);

		if (structure.getEntry_class() == null) {
			return map.size() > 1 ? map : processObjectByType(map.values().toArray()[0]);
		} else {
			return Bean.parse(structure.getEntry_class(), map);
		}
	}

	private Object processObjectByType(Object object) {
		if (object instanceof Long) {

			long obj = (long) object;

			if (obj < Integer.MAX_VALUE) {
				return (int) obj;
			}
		}

		return object;
	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private List<Map<String, Object>> queryForMaps(String sql, JdbcTemplate template, Collection<Object> paramers) {
		try {
			return template.queryForList(sql, paramers.toArray());
		} catch (EmptyResultDataAccessException e) {
			return null;
		}

	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Map<String, Object> queryForMap(String sql, JdbcTemplate template, Collection<Object> paramers) {
		try {
			return template.queryForMap(sql, paramers.toArray());
		} catch (EmptyResultDataAccessException e) {
			return null;
		}

	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForString(String sql, JdbcTemplate template, Collection<Object> paramers) {
		SqlRowSet rs = template.queryForRowSet(sql, paramers.toArray());
		while (rs.next()) {
			return rs.getString(1);
		}
		return null;
	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForInt(String sql, JdbcTemplate template, Collection<Object> paramers) {
		SqlRowSet rs = template.queryForRowSet(sql, paramers.toArray());
		while (rs.next()) {
			return rs.getInt(1);
		}
		return -1;
	}

}

==> cdm-core-2.0.1-RELEASE-sources.jar: online/sanen/cdm/handel/ResultHandel.java <==
package online.sanen.cdm.handel;

import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.SqlTypeValue;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import com.mhdt.toolkit.Bean;
import com.mhdt.toolkit.Collections;

import online.sanen.cdm.basic.QueryException;
import online.sanen.cdm.basic.QueryType;
import online.sanen.cdm.basic.ResultType;
import online.sanen.cdm.basic.Structure;

/**
 * The execution result
 * 
 * <pre>
 * &#64;author LazyToShow
 * 
 * Date: 2017/10/21 
 * Time: 23:19
 * </pre>
 */
public class ResultHandel extends DefaultHandel {

	@Override
	public Object handel(Structure structure, Object product) throws QueryException {

		QueryType queryType = structure.getQueryType();

		// Modify the operating
		if (queryType != QueryType.select) {
			if (structure.isCache())
				noticeUpdate(structure);
			return update(queryType, structure);
		}

		// Initialize the underlying data
		String sql = structure.getSql().toString();
		Collection<Object> paramers = structure.getParamers().values();
		JdbcTemplate template = (JdbcTemplate) structure.getTemplate();

		// Query operation
		ResultType resultType = structure.getResultType();

		Object result = null;

		// Try to read the results from the cache
		if (structure.isCache() && (result = tryReadFromCache(structure)) != null) {
			return processCacheResult(result, resultType);
		}

		try {
			switch (resultType) {

			case Int:
				result = queryForInt(sql, template, paramers);
				break;
			case String:
				result = queryForString(sql, template, paramers);
				break;
			case List:
				result = queryForList(structure, sql, template, paramers);
				break;
			case Object:
				result = queryForObject(structure, sql, template, paramers);
				break;
			case Bean:
				result = queryForBean(structure, sql, template, paramers);
				break;
			case Beans:
				result = queryForBeans(structure, sql, template, paramers);
				break;
			}

			// The result set requires manual limitations ?
			result = processLimit(result, structure);
			// Add the select cache
			if (structure.isCache())
				noticeAdd(structure, result);

			return result;

		} catch (Exception e) {
			throw new QueryException(e.getMessage() + formatSql(structure), e);
		}

	}

	private Object update(QueryType queryType, Structure structure) {
		String sql = structure.getSql().toString();
		Object[] paramers = structure.getParamers().values().toArray();
		JdbcTemplate template = (JdbcTemplate) structure.getTemplate();

		if (queryType.equals(QueryType.insert)) {
			
			KeyHolder keyHolder = new GeneratedKeyHolder();
			
			template.update(con->{
				PreparedStatement ps = con.prepareStatement(sql, PreparedStatement.RETURN_GENERATED_KEYS);
				for (int i = 0; i < paramers.length; i++) {
					if (paramers[i] == null)
						ps.setNull(i + 1, SqlTypeValue.TYPE_UNKNOWN);
					else
						ps.setObject(i + 1, paramers[i]);
				}
				return ps;
			}, keyHolder);
			
			
			Optional<Number> optional = Optional.ofNullable(keyHolder.getKey());  
			return optional.isPresent()?optional.get().intValue():1;
		} else {
			return template.update(sql, paramers);
		}
	}

	/**
	 * Resolve different problems with the same SQL result set<br>
	 * For example, the first query result is a list, and the second query result is
	 * object The second result is to be processed in the cache result
	 * 
	 * @param result
	 * @param resultType
	 * @return
	 */
	private Object processCacheResult(Object result, ResultType resultType) {

		if (result instanceof List && (resultType.equals(ResultType.Object) || resultType.equals(ResultType.Bean))) {
			return ((List<?>) result).get(0);
		}

		return result;
	}

	/**
	 * The result and the second limit,SQL that is not supported by the database
	 * Here we are implementing the interface manually
	 * 
	 * @param result
	 * @param structure
	 */
	private Object processLimit(Object result, Structure structure) {
		// If result is not list type,need not process
		if (!(result instanceof List))
			return result;

		// If there is a limit operation but SQL does not support, simulate limit
		if (structure.hasLimitAble() && !structure.isSupportLimitAble()) {

			// If the limit second parameter is null converted to -1, the number of access
			// is not limited
			result = Collections.limit((List<?>) result, structure.getLimit()[0],
					structure.getLimit()[1] == null ? -1 : structure.getLimit()[1]);
		}

		return result;
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForList(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		List<Map<String, Object>> list = queryForMaps(sql, template, paramers);

		if (list == null || list.isEmpty())
			return null;

		if (structure.getEntry_class() == null) {
			return processList(list);
		} else {
			return Bean.parse(structure.getEntry_class(), list);
		}
	}

	private Object processList(List<Map<String, Object>> list) {
		if (list.get(0).keySet().size() > 1)
			return list;
		List<Object> result = new ArrayList<Object>();
		for (Map<String, Object> map : list) {
			result.add(map.values().toArray()[0]);
		}

		return result;
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForBeans(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		List<Map<String, Object>> list = queryForMaps(sql, template, paramers);
		if (list == null || list.isEmpty())
			return null;

		return Bean.parse(structure.getEntry_class(), list);
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForBean(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		Map<String, Object> map = queryForMap(sql, template, paramers);
		if (map == null || map.isEmpty())
			return null;
		return Bean.parse(structure.getEntry_class(), map);
	}

	/**
	 * 
	 * @param structure
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForObject(Structure structure, String sql, JdbcTemplate template, Collection<Object> paramers) {
		List<Map<String, Object>> maps = queryForMaps(sql, template, paramers);
		if (maps == null || maps.isEmpty())
			return null;

		Map<String, Object> map = null;
		if (maps.size() > 0)
			map = maps.get(0);

		if (structure.getEntry_class() == null) {
			return map.size() > 1 ? map : processObjectByType(map.values().toArray()[0]);
		} else {
			return Bean.parse(structure.getEntry_class(), map);
		}
	}

	private Object processObjectByType(Object object) {
		if (object instanceof Long) {

			long obj = (long) object;

			if (obj < Integer.MAX_VALUE) {
				return (int) obj;
			}
		}

		return object;
	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private List<Map<String, Object>> queryForMaps(String sql, JdbcTemplate template, Collection<Object> paramers) {
		try {
			return template.queryForList(sql, paramers.toArray());
		} catch (EmptyResultDataAccessException e) {
			return null;
		}

	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Map<String, Object> queryForMap(String sql, JdbcTemplate template, Collection<Object> paramers) {
		try {
			return template.queryForMap(sql, paramers.toArray());
		} catch (EmptyResultDataAccessException e) {
			return null;
		}

	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForString(String sql, JdbcTemplate template, Collection<Object> paramers) {
		SqlRowSet rs = template.queryForRowSet(sql, paramers.toArray());
		while (rs.next()) {
			return rs.getString(1);
		}
		return null;
	}

	/**
	 * 
	 * @param sql
	 * @param template
	 * @param paramers
	 * @return
	 */
	private Object queryForInt(String sql, JdbcTemplate template, Collection<Object> paramers) {
		SqlRowSet rs = template.queryForRowSet(sql, paramers.toArray());
		while (rs.next()) {
			return rs.getInt(1);
		}
		return -1;
	}

}
